@@ -4,28 +4,49 @@
 // Copyright: 2021, Joylei <leingliu@gmail.com>
 // License: MIT
 
-use impl_trait_for_tuples::impl_for_tuples;
+#![allow(non_snake_case)]
+
+use std::marker::PhantomData;
 
 ///  generates output values based on its timing progress
 ///
 /// see [`crate::Timeline`]
+///
+/// Types derives [`Animatable`]:
+/// - `bool`
+/// - `i8`
+/// - `u8`
+/// - `i16`
+/// - `u16`
+/// - `i32`
+/// - `u32`
+/// - `i64`
+/// - `u64`
+/// - `f32`
+/// - `f64`
+/// - `i128`
+/// - `u168`
+/// - `Unit`
+/// - `Tuple`
+/// - `char`
+/// - `Option<T>` where `T:Animatable`
+/// - `PhantomData<T>`
+/// - `[T;N]` where `T:Animatable`
 pub trait Animatable: Sized + Clone {
     /// generates output values based on its timing progress
     fn animate(&self, to: &Self, time: f64) -> Self;
 }
 
 //-------- primitives -----------
-
-#[doc(hidden)]
 macro_rules! impl_primitive {
-    ($t:ident) => {
-        impl Animatable for $t {
+    ($ty:ident) => {
+        impl Animatable for $ty {
             #[inline]
             fn animate(&self, to: &Self, time: f64) -> Self {
                 if time == 0.0 {
                     return *self;
                 }
-                if time == 1.0 {
+                if (1.0 - time).abs() < f64::EPSILON {
                     return *to;
                 }
                 if self == to {
@@ -41,17 +62,17 @@ macro_rules! impl_primitive {
             }
         }
     };
-    ($t:ident, float) => {
-        impl Animatable for $t {
+    ($ty:ident, float) => {
+        impl Animatable for $ty {
             #[inline]
             fn animate(&self, to: &Self, time: f64) -> Self {
                 if time == 0.0 {
                     return *self;
                 }
-                if time == 1.0 {
+                if (1.0 - time).abs() < f64::EPSILON {
                     return *to;
                 }
-                if self == to {
+                if (self - to).abs() < $ty::EPSILON {
                     return *self;
                 }
                 crate::utils::check_time(time);
@@ -79,6 +100,7 @@ impl_primitive!(f32, float);
 impl_primitive!(f64, float);
 
 impl Animatable for bool {
+    #[inline]
     fn animate(&self, to: &Self, time: f64) -> Self {
         if time < 1.0 {
             *self
@@ -89,6 +111,7 @@ impl Animatable for bool {
 }
 
 impl Animatable for char {
+    #[inline]
     fn animate(&self, to: &Self, time: f64) -> Self {
         if self == to {
             return *self;
@@ -110,17 +133,77 @@ impl Animatable for char {
     }
 }
 
-//-------- tuples -----------
+impl Animatable for () {
+    #[inline]
+    fn animate(&self, _to: &Self, _time: f64) -> Self {}
+}
 
-#[impl_for_tuples(1, 10)]
-impl Animatable for Tuple {
-    for_tuples!( where #( Tuple: Animatable )* );
-
-    fn animate(&self, to: &Self, time: f64) -> Self {
-        for_tuples!( (#( Tuple::animate(&self.Tuple, &to.Tuple, time) ),* ))
+impl<T> Animatable for PhantomData<T> {
+    #[inline]
+    fn animate(&self, _to: &Self, _time: f64) -> Self {
+        Default::default()
     }
 }
 
+impl<T: Animatable> Animatable for Option<T> {
+    #[inline]
+    fn animate(&self, to: &Self, time: f64) -> Self {
+        match (self, to) {
+            (Some(a), Some(b)) => Some(a.animate(b, time)),
+            _ => None,
+        }
+    }
+}
+
+impl<T: Animatable, const N: usize> Animatable for [T; N] {
+    #[inline]
+    fn animate(&self, to: &Self, time: f64) -> Self {
+        let mut res = self.clone();
+        self.iter()
+            .zip(to.iter())
+            .zip(res.iter_mut())
+            .for_each(|((a, b), c)| *c = a.animate(b, time));
+        res
+    }
+}
+
+//-------- tuples -----------
+
+macro_rules! impl_tuple {
+    ($($n:tt $name:ident)+) => {
+        impl<'de, $($name,)+> Animatable for ($($name,)+)
+        where
+            $($name: Animatable,)+
+        {
+            #[inline]
+            fn animate(&self, to: &Self, time: f64) -> Self
+            {
+                $(
+                    let $name = Animatable::animate(&self.$n, &to.$n, time);
+                )+
+                ($($name,)+)
+            }
+        }
+    }
+}
+
+impl_tuple!(0 T0);
+impl_tuple!(0 T0 1 T1);
+impl_tuple!(0 T0 1 T1 2 T2);
+impl_tuple!(0 T0 1 T1 2 T2 3 T3);
+impl_tuple!(0 T0 1 T1 2 T2 3 T3 4 T4);
+impl_tuple!(0 T0 1 T1 2 T2 3 T3 4 T4 5 T5);
+impl_tuple!(0 T0 1 T1 2 T2 3 T3 4 T4 5 T5 6 T6);
+impl_tuple!(0 T0 1 T1 2 T2 3 T3 4 T4 5 T5 6 T6 7 T7);
+impl_tuple!(0 T0 1 T1 2 T2 3 T3 4 T4 5 T5 6 T6 7 T7 8 T8);
+impl_tuple!(0 T0 1 T1 2 T2 3 T3 4 T4 5 T5 6 T6 7 T7 8 T8 9 T9);
+impl_tuple!(0 T0 1 T1 2 T2 3 T3 4 T4 5 T5 6 T6 7 T7 8 T8 9 T9 10 T10);
+impl_tuple!(0 T0 1 T1 2 T2 3 T3 4 T4 5 T5 6 T6 7 T7 8 T8 9 T9 10 T10 11 T11);
+impl_tuple!(0 T0 1 T1 2 T2 3 T3 4 T4 5 T5 6 T6 7 T7 8 T8 9 T9 10 T10 11 T11 12 T12);
+impl_tuple!(0 T0 1 T1 2 T2 3 T3 4 T4 5 T5 6 T6 7 T7 8 T8 9 T9 10 T10 11 T11 12 T12 13 T13);
+impl_tuple!(0 T0 1 T1 2 T2 3 T3 4 T4 5 T5 6 T6 7 T7 8 T8 9 T9 10 T10 11 T11 12 T12 13 T13 14 T14);
+impl_tuple!(0 T0 1 T1 2 T2 3 T3 4 T4 5 T5 6 T6 7 T7 8 T8 9 T9 10 T10 11 T11 12 T12 13 T13 14 T14 15 T15);
+
 #[cfg(test)]
 mod test {
     use crate::Animatable;
