@@ -23,7 +23,7 @@ where
     Source: Animation,
     F: Fn(Source::Item) -> T,
 {
-    #[inline(always)]
+    #[inline]
     pub(super) fn new(src: Source, f: F) -> Self {
         Self { src, f }
     }
@@ -36,12 +36,12 @@ where
 {
     type Item = T;
 
-    #[inline(always)]
+    #[inline]
     fn duration(&self) -> Option<Duration> {
         self.src.duration()
     }
 
-    #[inline(always)]
+    #[inline]
     fn animate(&self, elapsed: Duration) -> Self::Item {
         let v = self.src.animate(elapsed);
         (self.f)(v)
